@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,8 +33,8 @@ from yapf.yapflib import split_penalty
 from yapf.yapflib import style
 from yapf.yapflib import unwrapped_line
 
-_COMPOUND_STMTS = frozenset({'for', 'while', 'if', 'elif', 'with', 'except',
-                             'def', 'class'})
+_COMPOUND_STMTS = frozenset(
+    {'for', 'while', 'if', 'elif', 'with', 'except', 'def', 'class'})
 
 
 class FormatDecisionState(object):
@@ -79,6 +79,7 @@ class FormatDecisionState(object):
     self.first_indent = first_indent
     self.newline = False
     self.previous = None
+    self.column_limit = style.Get('COLUMN_LIMIT')
     self._MoveStateToNextToken()
 
   def Clone(self):
@@ -94,9 +95,9 @@ class FormatDecisionState(object):
             self.column == other.column and
             self.paren_level == other.paren_level and
             self.start_of_line_level == other.start_of_line_level and
-            self.lowest_level_on_line == other.lowest_level_on_line and
-            (self.ignore_stack_for_comparison or
-             other.ignore_stack_for_comparison or self.stack == other.stack))
+            self.lowest_level_on_line == other.lowest_level_on_line and (
+                self.ignore_stack_for_comparison or
+                other.ignore_stack_for_comparison or self.stack == other.stack))
 
   def __ne__(self, other):
     return not self == other
@@ -123,7 +124,9 @@ class FormatDecisionState(object):
     """Returns True if the line must split before the next token."""
     current = self.next_token
     previous = current.previous_token
-    column_limit = style.Get('COLUMN_LIMIT')
+
+    if current.is_pseudo_paren:
+      return False
 
     if current.must_break_before:
       return True
@@ -143,8 +146,7 @@ class FormatDecisionState(object):
           else:
             last_token = _LastTokenInLine(bracket.matching_bracket)
 
-          length = last_token.total_length - bracket.total_length
-          if length + self.column >= column_limit:
+          if not self._FitsOnLine(bracket, last_token):
             self.stack[-1].split_before_closing_bracket = True
             return True
 
@@ -162,13 +164,20 @@ class FormatDecisionState(object):
     if (format_token.Subtype.DICTIONARY_KEY in current.subtypes and
         not current.is_comment):
       # Place each dictionary entry on its own line.
+      if previous.value == '{' and previous.previous_token:
+        opening = _GetOpeningBracket(previous.previous_token)
+        if (opening and opening.value == '(' and opening.previous_token and
+            opening.previous_token.is_name):
+          # This is a dictionary that's an argument to a function.
+          if self._FitsOnLine(previous, previous.matching_bracket):
+            return False
       return True
 
     # TODO(morbo): This should be controlled with a knob.
     if format_token.Subtype.DICT_SET_GENERATOR in current.subtypes:
       return True
 
-    if (style.Get('SPLIT_BEFORE_NAMED_ASSIGNS') and
+    if (style.Get('SPLIT_BEFORE_NAMED_ASSIGNS') and not current.is_comment and
         format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN_ARG_LIST in
         current.subtypes):
       if (previous.value not in {'=', ':', '*', '**'} and
@@ -183,40 +192,80 @@ class FormatDecisionState(object):
           #  a(
           #      b=1,
           #      c=2)
-          indent_amt = self.stack[-1].indent * style.Get('INDENT_WIDTH')
-          pptoken = previous.previous_token
-          opening_column = len(pptoken.value) if pptoken else 0 - indent_amt - 1
           if previous.value == '(':
-            return opening_column >= style.Get('CONTINUATION_INDENT_WIDTH')
-          opening = _GetOpeningParen(current)
+            if (self._FitsOnLine(previous, previous.matching_bracket) and
+                unwrapped_line.IsSurroundedByBrackets(previous)):
+              # An argument to a function is a function call with named
+              # assigns.
+              return False
+
+            func_start = previous.previous_token
+            while func_start and func_start.previous_token:
+              prev = func_start.previous_token
+              if not prev.is_name and prev.value != '.':
+                break
+              func_start = prev
+
+            if func_start:
+              func_name_len = previous.total_length - func_start.total_length
+              func_name_len += len(func_start.value)
+              return func_name_len > style.Get('CONTINUATION_INDENT_WIDTH')
+
+          opening = _GetOpeningBracket(current)
           if opening:
             arglist_length = (opening.matching_bracket.total_length -
                               opening.total_length + self.stack[-1].indent)
-            return arglist_length > column_limit
+            return arglist_length > self.column_limit
 
     if style.Get('SPLIT_ARGUMENTS_WHEN_COMMA_TERMINATED'):
       # Split before arguments in a function call or definition if the
       # arguments are terminated by a comma.
-      opening = _GetOpeningParen(current)
+      opening = _GetOpeningBracket(current)
       if opening and opening.previous_token and opening.previous_token.is_name:
         if previous.value in '(,':
           if opening.matching_bracket.previous_token.value == ',':
             return True
 
-    if (previous.value in '{[' and current.lineno != previous.lineno and
-        format_token.Subtype.SUBSCRIPT_BRACKET not in previous.subtypes):
-      # Retain the split after the container opening.
-      return True
+    if (format_token.Subtype.DICTIONARY_VALUE in current.subtypes or
+        (previous.is_pseudo_paren and previous.value == '(')):
+      if not current.OpensScope():
+        opening = _GetOpeningBracket(current)
+        if previous.is_pseudo_paren:
+          # Split before the dictionary value if we can't fit the whole
+          # dictionary on one line.
+          if not self._EachDictEntryFitsOnOneLine(opening):
+            return True
 
-    if (previous.value == ':' and _IsDictionaryValue(current) and
-        current.lineno != previous.lineno):
-      # Retain the split between the dictionary key and value.
-      return True
+    if (previous.OpensScope() and not current.OpensScope() and
+        format_token.Subtype.SUBSCRIPT_BRACKET not in previous.subtypes):
+
+      if not current.is_comment:
+        pprevious = previous.previous_token
+        if pprevious and not pprevious.is_keyword and not pprevious.is_name:
+          # We want to split if there's a comment in the container.
+          token = current
+          while token != previous.matching_bracket:
+            if token.is_comment:
+              return True
+            token = token.next_token
+
+      if previous.value == '(':
+        pptoken = previous.previous_token
+        if not pptoken or not pptoken.is_name:
+          # Split after the opening of a tuple if it doesn't fit on the current
+          # line and it's not a function call.
+          if self._FitsOnLine(previous, previous.matching_bracket):
+            return False
+      else:
+        # Split after the opening of a container if it doesn't fit on the
+        # current line or if it has a comment.
+        if not self._FitsOnLine(previous, previous.matching_bracket):
+          return True
 
     if previous.value == '{':
       closing = previous.matching_bracket
-      length = closing.total_length - previous.total_length + self.column
-      if length > column_limit and closing.previous_token.value == ',':
+      if (not self._FitsOnLine(previous, closing) and
+          closing.previous_token.value == ','):
         self.stack[-1].split_before_closing_bracket = True
         return True
 
@@ -225,22 +274,20 @@ class FormatDecisionState(object):
       # Split at the beginning of a list comprehension.
       length = _GetLengthOfSubtype(current, format_token.Subtype.COMP_FOR,
                                    format_token.Subtype.COMP_IF)
-      if length + self.column > column_limit:
+      if length + self.column > self.column_limit:
         return True
 
     if (format_token.Subtype.COMP_IF in current.subtypes and
         format_token.Subtype.COMP_IF not in previous.subtypes):
       # Split at the beginning of an if expression.
       length = _GetLengthOfSubtype(current, format_token.Subtype.COMP_IF)
-      if length + self.column > column_limit:
+      if length + self.column > self.column_limit:
         return True
 
     previous_previous_token = previous.previous_token
     if (current.is_name and previous_previous_token and
         previous_previous_token.is_name and previous.value == '('):
-      arg_length = previous.matching_bracket.total_length
-      arg_length -= previous.total_length
-      if arg_length + self.column > column_limit:
+      if not self._FitsOnLine(previous, previous.matching_bracket):
         if _IsFunctionCallWithArguments(current):
           # There is a function call, with more than 1 argument, where
           # the first argument is itself a function call with arguments.
@@ -250,9 +297,7 @@ class FormatDecisionState(object):
           # argument to keep things looking good.
           return True
         elif current.OpensScope():
-          arg_length = current.matching_bracket.total_length
-          arg_length -= current.total_length
-          if arg_length + self.column > column_limit:
+          if not self._FitsOnLine(current, current.matching_bracket):
             # There is a data literal that will need to be split and could mess
             # up the formatting.
             return True
@@ -269,28 +314,26 @@ class FormatDecisionState(object):
       # original comments were on a separate line.
       return True
 
-    if not current.next_token:
-      return False
-
-    opening = _GetOpeningParen(current)
-    if (opening and opening.value == '(' and current.is_name and
-        previous.value == ','):
+    if current.is_name and previous.value == ',':
       # If we have a function call within an argument list and it won't fit on
       # the remaining line, but it will fit on a line by itself, then go ahead
       # and split before the call.
-      total_len = 0
-      ntoken = current
-      while ntoken:
-        if ntoken.value == '(':
-          total_len = ntoken.matching_bracket.total_length - current.total_length
-          break
-        ntoken = ntoken.next_token
+      opening = _GetOpeningBracket(current)
+      if (opening and opening.value == '(' and opening.previous_token and
+          opening.previous_token.is_name):
+        is_func_call = False
+        token = current
+        while token:
+          if token.value == '(':
+            is_func_call = True
+            break
+          if not token.is_name and token.value != '.':
+            break
+          token = token.next_token
 
-      if ntoken:
-        indent_amt = self.stack[-1].indent * style.Get('INDENT_WIDTH')
-        if (total_len + indent_amt < column_limit and
-            total_len + self.column >= column_limit):
-          return True
+        if is_func_call:
+          if not self._FitsOnLine(current, opening.matching_bracket):
+            return True
 
     return False
 
@@ -377,9 +420,9 @@ class FormatDecisionState(object):
     self.start_of_line_level = self.paren_level
     self.lowest_level_on_line = self.paren_level
 
-    if (previous.OpensScope() or (previous.is_comment and
-                                  previous.previous_token is not None and
-                                  previous.previous_token.OpensScope())):
+    if (previous.OpensScope() or
+        (previous.is_comment and previous.previous_token is not None and
+         previous.previous_token.OpensScope())):
       self.stack[-1].closing_scope_indent = max(
           0, self.stack[-1].indent - style.Get('CONTINUATION_INDENT_WIDTH'))
       self.stack[-1].split_before_closing_bracket = True
@@ -418,9 +461,9 @@ class FormatDecisionState(object):
       return top_of_stack.indent if self.paren_level else self.first_indent
 
     if current.ClosesScope():
-      if (previous.OpensScope() or (previous.is_comment and
-                                    previous.previous_token is not None and
-                                    previous.previous_token.OpensScope())):
+      if (previous.OpensScope() or
+          (previous.is_comment and previous.previous_token is not None and
+           previous.previous_token.OpensScope())):
         return max(0,
                    top_of_stack.indent - style.Get('CONTINUATION_INDENT_WIDTH'))
       return top_of_stack.closing_scope_indent
@@ -479,16 +522,15 @@ class FormatDecisionState(object):
     if is_multiline_string:
       # This is a multiline string. Only look at the first line.
       self.column += len(current.value.split('\n')[0])
-    elif not current.is_pseudo_paren or current.value == '(':
+    elif not current.is_pseudo_paren:
       self.column += len(current.value)
 
     self.next_token = self.next_token.next_token
 
     # Calculate the penalty for overflowing the column limit.
     penalty = 0
-    column_limit = style.Get('COLUMN_LIMIT')
-    if not current.is_pylint_comment and self.column > column_limit:
-      excess_characters = self.column - column_limit
+    if not current.is_pylint_comment and self.column > self.column_limit:
+      excess_characters = self.column - self.column_limit
       penalty += style.Get('SPLIT_PENALTY_EXCESS_CHARACTER') * excess_characters
 
     if is_multiline_string:
@@ -498,6 +540,74 @@ class FormatDecisionState(object):
 
     return penalty
 
+  def _FitsOnLine(self, start, end):
+    """Determines if line between start and end can fit on the current line."""
+    length = end.total_length - start.total_length
+    if not start.is_pseudo_paren:
+      length += len(start.value)
+    return length + self.column <= self.column_limit
+
+  def _EachDictEntryFitsOnOneLine(self, opening):
+    """Determine if each dict elems can fit on one line."""
+
+    def PreviousNonCommentToken(tok):
+      tok = tok.previous_token
+      while tok.is_comment:
+        tok = tok.previous_token
+      return tok
+
+    def ImplicitStringConcatenation(tok):
+      num_strings = 0
+      if tok.is_pseudo_paren:
+        tok = tok.next_token
+      while tok.is_string:
+        num_strings += 1
+        tok = tok.next_token
+      return num_strings > 1
+
+    closing = opening.matching_bracket
+    entry_start = opening.next_token
+    current = opening.next_token.next_token
+
+    while current and current != closing:
+      if format_token.Subtype.DICTIONARY_KEY in current.subtypes:
+        prev = PreviousNonCommentToken(current)
+        length = prev.total_length - entry_start.total_length
+        length += len(entry_start.value)
+        if length + self.stack[-2].indent >= self.column_limit:
+          return False
+        entry_start = current
+      if current.OpensScope():
+        if ((current.value == '{' or
+             (current.is_pseudo_paren and current.next_token.value == '{') and
+             format_token.Subtype.DICTIONARY_VALUE in current.subtypes) or
+            ImplicitStringConcatenation(current)):
+          # A dictionary entry that cannot fit on a single line shouldn't matter
+          # to this calcuation. If it can't fit on a single line, then the
+          # opening should be on the same line as the key and the rest on
+          # newlines after it. But the other entries should be on single lines
+          # if possible.
+          if current.matching_bracket:
+            current = current.matching_bracket
+          while current:
+            if current == closing:
+              return True
+            if format_token.Subtype.DICTIONARY_KEY in current.subtypes:
+              entry_start = current
+              break
+            current = current.next_token
+        else:
+          current = current.matching_bracket
+      else:
+        current = current.next_token
+
+    # At this point, current is the closing bracket. Go back one to get the the
+    # end of the dictionary entry.
+    current = PreviousNonCommentToken(current)
+    length = current.total_length - entry_start.total_length
+    length += len(entry_start.value)
+    return length + self.stack[-2].indent <= self.column_limit
+
 
 def _IsFunctionCallWithArguments(token):
   while token:
@@ -510,14 +620,6 @@ def _IsFunctionCallWithArguments(token):
   return False
 
 
-def _IsDictionaryValue(token):
-  while token:
-    if format_token.Subtype.DICTIONARY_VALUE in token.subtypes:
-      return True
-    token = token.previous_token
-  return False
-
-
 def _GetLengthOfSubtype(token, subtype, exclude=None):
   current = token
   while (current.next_token and subtype in current.subtypes and
@@ -526,17 +628,18 @@ def _GetLengthOfSubtype(token, subtype, exclude=None):
   return current.total_length - token.total_length + 1
 
 
-def _GetOpeningParen(current):
-  previous = current
-  if previous and previous.matching_bracket:
-    return previous.matching_bracket
-  while previous is not None and previous.matching_bracket is None:
-    previous = previous.previous_token
-    if not previous:
-      break
-    if previous.ClosesScope():
-      previous = previous.matching_bracket.previous_token
-  return previous
+def _GetOpeningBracket(current):
+  if current.matching_bracket and not current.is_pseudo_paren:
+    return current.matching_bracket
+  while current:
+    if current.ClosesScope():
+      current = current.matching_bracket
+    elif current.is_pseudo_paren:
+      current = current.previous_token
+    elif current.OpensScope():
+      return current
+    current = current.previous_token
+  return None
 
 
 def _LastTokenInLine(current):
@@ -595,3 +698,13 @@ class _ParenState(object):
   def __repr__(self):
     return '[indent::%d, last_space::%d, closing_scope_indent::%d]' % (
         self.indent, self.last_space, self.closing_scope_indent)
+
+  def __eq__(self, other):
+    return hash(self) == hash(other)
+
+  def __ne__(self, other):
+    return not self == other
+
+  def __hash__(self, *args, **kwargs):
+    return hash((self.indent, self.last_space, self.closing_scope_indent,
+                 self.split_before_closing_bracket, self.num_line_splits))
