@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -75,21 +75,24 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
         comp_for = True
         _AppendFirstLeafTokenSubtype(child,
                                      format_token.Subtype.DICT_SET_GENERATOR)
-      elif pytree_utils.NodeName(child) == 'COLON':
+      elif pytree_utils.NodeName(child) in ('COLON', 'DOUBLESTAR'):
         dict_maker = True
 
     if not comp_for and dict_maker:
       last_was_colon = False
       for child in node.children:
         if dict_maker:
+          if pytree_utils.NodeName(child) == 'DOUBLESTAR':
+            _AppendFirstLeafTokenSubtype(child,
+                                         format_token.Subtype.KWARGS_STAR_STAR)
           if last_was_colon:
             if style.Get('INDENT_DICTIONARY_VALUE'):
               _InsertPseudoParentheses(child)
             else:
               _AppendFirstLeafTokenSubtype(
                   child, format_token.Subtype.DICTIONARY_VALUE)
-          elif (child is not None and
-                (isinstance(child, pytree.Node) or child.value not in '{:,')):
+          elif (child is not None and (isinstance(child, pytree.Node) or (
+              not child.value.startswith('#') and child.value not in '{:,'))):
             # Mark the first leaf of a key entry as a DICTIONARY_KEY. We
             # normally want to split before them if the dictionary cannot exist
             # on a single line.
@@ -239,6 +242,14 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
     self._ProcessArgLists(node)
     _SetDefaultOrNamedAssignArgListSubtype(node)
 
+  def Visit_decorator(self, node):  # pylint: disable=invalid-name
+    # decorator ::=
+    #     '@' dotted_name [ '(' [arglist] ')' ] NEWLINE
+    for child in node.children:
+      if isinstance(child, pytree.Leaf) and child.value == '@':
+        _AppendTokenSubtype(child, subtype=format_token.Subtype.DECORATOR)
+      self.Visit(child)
+
   def Visit_funcdef(self, node):  # pylint: disable=invalid-name
     # funcdef ::=
     #     'def' NAME parameters ['->' test] ':' suite
@@ -302,7 +313,8 @@ def _SetDefaultOrNamedAssignArgListSubtype(node):
       return False
     has_subtype = False
     for child in node.children:
-      has_subtype |= HasDefaultOrNamedAssignSubtype(child)
+      if pytree_utils.NodeName(child) != 'arglist':
+        has_subtype |= HasDefaultOrNamedAssignSubtype(child)
     return has_subtype
 
   if HasDefaultOrNamedAssignSubtype(node):
