@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -72,8 +72,8 @@ class UnwrappedLine(object):
           _SpaceRequiredBetween(prev_token, token)):
         token.spaces_required_before = 1
 
-      token.total_length = (
-          prev_length + len(token.value) + token.spaces_required_before)
+      tok_len = len(token.value) if not token.is_pseudo_paren else 0
+      token.total_length = prev_length + tok_len + token.spaces_required_before
 
       # The split penalty has to be computed before {must|can}_break_before,
       # because these may use it for their decision.
@@ -194,7 +194,10 @@ def _SpaceRequiredBetween(left, right):
     # Space between keyword... tokens and pseudo parens.
     return True
   if left.is_pseudo_paren or right.is_pseudo_paren:
-    # The pseudo-parens shouldn't affect spacing.
+    # There should be a space after the ':' in a dictionary.
+    if left.OpensScope():
+      return True
+    # The closing pseudo-paren shouldn't affect spacing.
     return False
   if left.is_continuation or right.is_continuation:
     # The continuation node's value has all of the spaces it needs.
@@ -227,10 +230,16 @@ def _SpaceRequiredBetween(left, right):
       (left.is_keyword or left.is_name)):
     # Don't merge two keywords/identifiers.
     return True
-  if left.is_string and rval not in '[)]}.':
-    # A string followed by something other than a subscript, closing bracket,
-    # or dot should have a space after it.
-    return True
+  if left.is_string:
+    if (rval == '=' and format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN_ARG_LIST in
+        right.subtypes):
+      # If there is a type hint, then we don't want to add a space between the
+      # equal sign and the hint.
+      return False
+    if rval not in '[)]}.':
+      # A string followed by something other than a subscript, closing bracket,
+      # or dot should have a space after it.
+      return True
   if left.is_binary_op and lval != '**' and _IsUnaryOperator(right):
     # Space between the binary opertor and the unary operator.
     return True
@@ -264,7 +273,7 @@ def _SpaceRequiredBetween(left, right):
       format_token.Subtype.KWARGS_STAR_STAR in left.subtypes):
     # Don't add a space after a vararg's star or a keyword's star-star.
     return False
-  if lval == '@':
+  if (lval == '@' and format_token.Subtype.DECORATOR in left.subtypes):
     # Decorators shouldn't be separated from the 'at' sign.
     return False
   if lval == '.' or rval == '.':
